TermCtx

Struct TermCtx 

pub struct TermCtx { /* private fields */ }
Available on crate feature rt only.
Context passed to term() service application callback.

This context can be used to query whether the service application is running as foreground process or running within a service subsystem.

It can also be used to report progress status back to the service subsystems, for platforms that support it.

Implementations

Source

impl TermCtx

pub fn runenv(&self) -> RunEnv

Return context used to identify whether service application is running as a foreground process or a system service.

pub fn report(&self, status: Option<&str>)

Report shutdown state to the system service manager.

For foreground processes and services that do not support shutdown state notifications this method has no effect.

pub fn get<T: 'static>(&self) -> Option<&T>

pub fn take<T: 'static>(&mut self) -> Option<T>

Extract passthrough buffer.